function    [outputVector1,...
             errorVector,...
             coefficientVector1] =   LMS1(desired,input1,step,filter_length,initial_coeff1)

%   LMS.m
%       Implements the Complex LMS algorithm for COMPLEX valued data.
%       (Algorithm 3.2 - book: Adaptive Filtering: Algorithms and Practical
%                                                       Implementation, Diniz)
%
%   Syntax:
%       [outputVector,errorVector,coefficientVector] = LMS(desired,input,S)
%
%   Input Arguments:
%       . desired   : Desired signal.                               (ROW vector)
%       . input     : Signal fed into the adaptive filter.          (ROW vector)
%       . S         : Structure with the following fields
%           - step                  : Convergence (relaxation) factor.
%           - filterOrderNo         : Order of the FIR filter.
%           - initialCoefficients   : Initial filter coefficients.  (COLUMN vector)
%
%   Output Arguments:
%       . outputVector      :   Store the estimated output of each iteration.   (COLUMN vector)
%       . errorVector       :   Store the error for each iteration.             (COLUMN vector)
%       . coefficientVector :   Store the estimated coefficients for each iteration.
%                               (Coefficients at one iteration are COLUMN vector)
%
%   Authors:
%       . Guilherme de Oliveira Pinto   - guilhermepinto7@gmail.com & guilherme@lps.ufrj.br
%       . Markus Vinícius Santos Lima   - mvsl20@gmailcom           & markus@lps.ufrj.br
%       . Wallace Alves Martins         - wallace.wam@gmail.com     & wallace@lps.ufrj.br
%       . Luiz Wagner Pereira Biscainho - cpneqs@gmail.com          & wagner@lps.ufrj.br
%       . Paulo Sergio Ramirez Diniz    -                             diniz@lps.ufrj.br
%


%   Some Variables and Definitions:
%       . prefixedInput         :   Input is prefixed by nCoefficients -1 zeros.
%                                   (The prefix led to a more regular source code)
%
%       . regressor             :   Auxiliar variable. Store the piece of the
%                                   prefixedInput that will be multiplied by the
%                                   current set of coefficients.
%                                   (regressor is a COLUMN vector)
%
%       . nCoefficients         :   FIR filter number of coefficients.
%
%       . nIterations           :   Number of iterations.


%   Initialization Procedure
nCoefficients       =   filter_length;
nIterations         =   length(desired);

%   Pre Allocations
errorVector             =   zeros(nIterations   ,1);
outputVector1            =   zeros(nIterations   ,1);

coefficientVector1       =   zeros(nCoefficients ,(nIterations+1));


%   Initial State Weight Vector
coefficientVector1(:,1)  =   initial_coeff1;
%   Improve source code regularity
prefixedInput1           =   [zeros(nCoefficients-1,1)
                             transpose(input1)];

%   Body
for it = 1:nIterations,
it;
    regressor1                   =   prefixedInput1(it+(nCoefficients-1):-1:it,1);
    

    outputVector1(it,1)          =   (coefficientVector1(:,it)')*regressor1;
   
    %outputVector(it,1)           =   outputVector1(it,1);
    errorVector(it,1)           =   desired(it)-outputVector1(it,1);

    coefficientVector1(:,it+1)   =   coefficientVector1(:,it)+...
                                    (step*conj(errorVector(it,1))*...
                                    regressor1);
                                
end
%errorVector;
abs(errorVector(it,1))
%regressor1  
%coefficientVector1(:,it+1)
%it
%   EOF
